@@ -19,15 +19,17 @@
 #include "structmember.h"
 #include "oputil.h"
 #include "iops.h"
+#include "python_wrappers.h"
 #include <libcouchbase/vbucket.h>
+#include <libcouchbase/error.h>
 
 PyObject *pycbc_DummyTuple;
 PyObject *pycbc_DummyKeywords;
 
 static int
 Bucket__init__(pycbc_Bucket *self,
-                   PyObject *args,
-                   PyObject *kwargs);
+               PyObject *args,
+               PyObject *kwargs);
 
 static PyObject*
 Bucket__connect(pycbc_Bucket *self, PyObject* args, PyObject* kwargs);
@@ -108,14 +110,15 @@ static PyObject*
 Bucket_register_crypto_provider(pycbc_Bucket *self, PyObject *args) {
     char *name = NULL;
     pycbc_CryptoProvider *provider = NULL;
-
+    lcb_STATUS err = LCB_SUCCESS;
     if (!PyArg_ParseTuple(args, "sO", &name, &provider)) {
         PYCBC_EXCTHROW_ARGS();
         return NULL;
     }
 
     if (!provider || !provider->lcb_provider) {
-        PYCBC_EXC_WRAP(PYCBC_EXC_LCBERR, LCB_EINVAL, "Invalid provider");
+        PYCBC_EXC_WRAP(
+                PYCBC_EXC_LCBERR, LCB_ERR_INVALID_ARGUMENT, "Invalid provider");
         return NULL;
     }
     {
@@ -132,27 +135,41 @@ Bucket_register_crypto_provider(pycbc_Bucket *self, PyObject *args) {
         PYCBC_XDECREF(ctor_args);
         if (named_provider_proxy && !PyErr_Occurred()) {
             PYCBC_INCREF(named_provider_proxy);
-            lcbcrypto_register(
+            err=pycbc_crypto_register(
                     self->instance, name, named_provider_proxy->lcb_provider);
+            if (err){
+                PYCBC_REPORT_ERR(err,"Can't register crypto provider");
+                goto GT_FAIL;
+            }
+            Py_RETURN_NONE;
         } else {
+            GT_FAIL:
             PYCBC_EXCEPTION_LOG_NOCLEAR;
             PYCBC_XDECREF(named_provider_proxy);
         }
+
     }
-    return Py_None;
+    return NULL;
 }
 
 static PyObject*
 Bucket_unregister_crypto_provider(pycbc_Bucket *self, PyObject *args)
 {
     char *name = NULL;
+    lcb_STATUS err=LCB_SUCCESS;
 
     if (!PyArg_ParseTuple(args, "s", &name)) {
         PYCBC_EXCTHROW_ARGS();
         return NULL;
     }
-    lcbcrypto_unregister(self->instance, name);
-    return Py_None;
+
+    err=pycbc_crypto_unregister(self->instance,name);
+    if (err)
+    {
+        PYCBC_REPORT_ERR(err,"Can't unregister crypto provider");
+        return NULL;
+    }
+    Py_RETURN_NONE;
 }
 
 const char *pycbc_dict_cstr(PyObject *dp, char *key) {
@@ -176,21 +193,17 @@ size_t pycbc_populate_fieldspec(lcbcrypto_FIELDSPEC **fields,
         PyObject *alg = PyDict_GetItemString(dict, "alg");
         if (!alg) {
             PYCBC_EXC_WRAP(
-                    PYCBC_EXC_ARGUMENTS, PYCBC_CRYPTO_PROVIDER_ALIAS_NULL, "")
+                    PYCBC_EXC_ARGUMENTS, (lcb_STATUS)PYCBC_CRYPTO_PROVIDER_ALIAS_NULL, "")
             goto FAIL;
         }
         (*fields)[i].alg = pycbc_dict_cstr(dict, "alg");
-#if PYCBC_CRYPTO_VERSION < 1
-        (*fields)[i].kid = kid_value;
-#else
         if (kid_value && strlen(kid_value) > 0) {
             PYCBC_EXC_WRAP(PYCBC_EXC_LCBERR,
-                           LCB_EINVAL,
+                           LCB_ERR_INVALID_ARGUMENT,
                            "Fieldspec should not include Key ID - this should "
                            "be provided by get_key_id instead");
             goto FAIL;
         }
-#endif
         (*fields)[i].name = pycbc_dict_cstr(dict, "name");
     }
     return size;
@@ -205,7 +218,7 @@ Bucket_encrypt_fields(pycbc_Bucket *self, PyObject *args)
 {
     lcbcrypto_CMDENCRYPT cmd = {0};
     PyObject* fieldspec=NULL;
-    int res =0;
+    int res = 0;
     PyObject *result = NULL;
     if (!PyArg_ParseTuple(args, "s#Os", &cmd.doc, &cmd.ndoc, &fieldspec, &(cmd.prefix))) {
         PYCBC_EXCTHROW_ARGS();
@@ -215,14 +228,11 @@ Bucket_encrypt_fields(pycbc_Bucket *self, PyObject *args)
     if (!PyErr_Occurred()) {
         cmd.nfields = pycbc_populate_fieldspec(&cmd.fields, fieldspec);
     }
-    if (!PyErr_Occurred()) {
-#if PYCBC_CRYPTO_VERSION > 0
-        res = lcbcrypto_encrypt_fields(self->instance, &cmd);
-#else
-        res = lcbcrypto_encrypt_document(self->instance, &cmd);
-#endif
-
+    if (PyErr_Occurred()) {
+        goto FINISH;
     }
+    res = pycbc_encrypt_fields(self->instance, &cmd);
+
     if (PyErr_Occurred()) {
         goto FINISH;
     }
@@ -230,9 +240,7 @@ Bucket_encrypt_fields(pycbc_Bucket *self, PyObject *args)
         result = pycbc_SimpleStringN(cmd.out, cmd.nout);
     }
     if (!result && !PyErr_Occurred()) {
-        PYCBC_EXC_WRAP(PYCBC_EXC_LCBERR,
-                       LCB_GENERIC_TMPERR,
-                       "Internal error while encrypting");
+        PYCBC_REPORT_ERR(res, "Internal error while encrypting");
     }
 FINISH:
     PYCBC_FREE(cmd.out);
@@ -244,42 +252,32 @@ static PyObject *
 Bucket_decrypt_fields(pycbc_Bucket *self, PyObject *args)
 {
     lcbcrypto_CMDDECRYPT cmd = { 0 };
-    int res =0;
+    int res = 0;
     PyObject* fieldspec = NULL;
     PyObject *result = NULL;
     if (!PyArg_ParseTuple(args, "s#"
-#if PYCBC_CRYPTO_VERSION >0
                                 "O"
-#endif
                                 "s", &cmd.doc, &cmd.ndoc,
-#if PYCBC_CRYPTO_VERSION > 0
                           &fieldspec,
-#endif
                           &cmd.prefix)) {
         PYCBC_EXCTHROW_ARGS();
         return NULL;
     }
 
     if (!PyErr_Occurred()) {
-#if PYCBC_CRYPTO_VERSION > 0
         cmd.nfields = pycbc_populate_fieldspec(&cmd.fields, fieldspec);
-        res = lcbcrypto_decrypt_fields(self->instance, &cmd);
-#else
-        (void)fieldspec;
-        res = lcbcrypto_decrypt_document(self->instance, &cmd);
-#endif
+        res = pycbc_decrypt_fields(self->instance, &cmd);
     }
 
     if (PyErr_Occurred()) {
         goto FINISH;
     }
+
     if (!res) {
         result = pycbc_SimpleStringN(cmd.out, cmd.nout);
     }
     if (!result && !PyErr_Occurred()) {
-        PYCBC_EXC_WRAP(PYCBC_EXC_LCBERR,
-                       LCB_GENERIC_TMPERR,
-                       "Internal error while decrypting");
+        PYCBC_REPORT_ERR(res, "Internal error while encrypting");
     }
 FINISH:
     PYCBC_FREE(cmd.out);
@@ -292,6 +290,11 @@ Bucket_server_nodes(pycbc_Bucket *self, void *unused)
     const char * const *cnodes;
     const char **curnode;
     PyObject *ret_list;
+    if (!self->instance)
+    {
+        Py_RETURN_NONE;
+    }
+
     cnodes = lcb_get_server_list(self->instance);
 
     if (!cnodes) {
@@ -318,7 +321,11 @@ Bucket_server_nodes(pycbc_Bucket *self, void *unused)
 static PyObject *
 Bucket_get_configured_replica_count(pycbc_Bucket *self, void *unused)
 {
-    PyObject *iret = pycbc_IntFromUL(lcb_get_num_replicas(self->instance));
+    PyObject *iret = self->instance?pycbc_IntFromUL(lcb_get_num_replicas(self->instance)):NULL;
+    if (!iret)
+    {
+        Py_RETURN_NONE;
+    }
 
     (void)unused;
     return iret;
@@ -331,7 +338,12 @@ Bucket_connected(pycbc_Bucket *self, void *unused)
 
     if (ret == Py_False) {
         void *handle = NULL;
-        lcb_error_t err;
+        lcb_STATUS err=LCB_SUCCESS;
+        if (!self->instance)
+        {
+            Py_RETURN_FALSE;
+        }
+
         err = lcb_cntl(self->instance, LCB_CNTL_GET, LCB_CNTL_VBCONFIG, &handle);
         if (err == LCB_SUCCESS && handle != NULL) {
             self->flags |= PYCBC_CONN_F_CONNECTED;
@@ -346,14 +358,13 @@ Bucket_connected(pycbc_Bucket *self, void *unused)
     return ret;
 }
 
-#ifdef PYCBC_TRACING
 static PyObject *Bucket_tracer(pycbc_Bucket *self, void *unused)
 {
     PyObject *result = self->tracer ? (PyObject *)self->tracer : Py_None;
     PYCBC_INCREF(result);
     return result;
 }
-#endif
+
 static PyObject *
 Bucket__instance_pointer(pycbc_Bucket *self, void *unused)
 {
@@ -368,7 +379,7 @@ static PyObject *
 Bucket__add_creds(pycbc_Bucket *self, PyObject *args)
 {
     char *arr[2] = { NULL };
-    lcb_error_t rc;
+    lcb_STATUS rc=LCB_SUCCESS;
     if (!PyArg_ParseTuple(args, "ss", &arr[0], &arr[1])) {
         return NULL;
     }
@@ -411,17 +422,15 @@ Bucket__thr_lockop(pycbc_Bucket *self, PyObject *arg)
 static PyObject *
 Bucket__close(pycbc_Bucket *self)
 {
-    lcb_error_t err;
+    lcb_STATUS err=LCB_SUCCESS;
 
     if (self->flags & PYCBC_CONN_F_CLOSED) {
         Py_RETURN_NONE;
     }
 
     self->flags |= PYCBC_CONN_F_CLOSED;
-#ifdef PYCBC_TRACING
     Py_XDECREF(self->tracer);
     self->tracer = NULL;
-#endif
     lcb_destroy(self->instance);
 
     if (self->iopswrap) {
@@ -501,13 +510,13 @@ Bucket__get_timings(pycbc_Bucket *self)
     return ll;
 }
 
+
 static PyObject *
-Bucket__mutinfo(pycbc_Bucket *self)
-{
+Bucket__mutinfo(pycbc_Bucket *self) {
     PyObject *ll = PyList_New(0);
     size_t ii, vbmax;
     lcbvb_CONFIG *cfg = NULL;
-    lcb_error_t rc;
+    lcb_STATUS rc = LCB_SUCCESS;
 
     rc = lcb_cntl(self->instance, LCB_CNTL_GET, LCB_CNTL_VBCONFIG, &cfg);
     if (rc != LCB_SUCCESS) {
@@ -516,26 +525,40 @@ Bucket__mutinfo(pycbc_Bucket *self)
     }
 
     vbmax = vbucket_config_get_num_vbuckets(cfg);
+    PYCBC_DEBUG_LOG("Got %d buckets", vbmax)
     for (ii = 0; ii < vbmax; ++ii) {
-        lcb_KEYBUF kb = { 0 };
+        PYCBC_DEBUG_LOG("Examining vbucket %d", ii)
+        lcb_KEYBUF kb = {0};
         const lcb_MUTATION_TOKEN *mt;
-        lcb_error_t rc = LCB_SUCCESS;
+        lcb_STATUS rc = LCB_SUCCESS;
         PyObject *cur;
 
         kb.type = LCB_KV_VBID;
-        kb.contig.nbytes = (size_t)ii;
+        kb.contig.nbytes = (size_t) ii;
 
-        mt = lcb_get_mutation_token(self->instance, &kb, &rc);
-        if (mt == NULL) {
+        mt = pycbc_get_vbucket_mutation_token(self->instance, &kb, &rc);
+        if (rc == LCB_ERR_UNSUPPORTED_OPERATION) {
+            PYCBC_EXC_WRAP(PYCBC_EXC_LCBERR, rc, "Mutation token info on VBucket not supported")
+            goto GT_FAIL;
+        }
+        if (mt == NULL ) {
+            PYCBC_DEBUG_LOG("No mutation token for vbucket %d", ii)
             continue;
         }
-        cur = Py_BuildValue("HKK", LCB_MUTATION_TOKEN_VB(mt),
-            LCB_MUTATION_TOKEN_ID(mt), LCB_MUTATION_TOKEN_SEQ(mt));
+
+        cur = Py_BuildValue("HKK",
+                            pycbc_mutation_token_vbid(mt),
+                            pycbc_mutation_token_uuid(mt),
+                            pycbc_mutation_token_seqno(mt));
+        PYCBC_DEBUG_PYFORMAT("Got mutinfo %R", cur)
+        PYCBC_EXCEPTION_LOG_NOCLEAR
         PyList_Append(ll, cur);
         Py_DECREF(cur);
     }
-
     return ll;
+    GT_FAIL:
+    PYCBC_DECREF(ll);
+    return NULL;
 }
 
 static PyGetSetDef Bucket_TABLE_getset[] = {
@@ -559,11 +582,11 @@ static PyGetSetDef Bucket_TABLE_getset[] = {
         { "transcoder",
                 (getter)Bucket_get_transcoder,
                 (setter)Bucket_set_transcoder,
-                PyDoc_STR("The :class:`~couchbase.transcoder.Transcoder` "
+                PyDoc_STR("The :class:`~couchbase_core.transcoder.Transcoder` "
                         "object being used.\n\n"
                         ""
                         "This is normally ``None`` unless a custom "
-                        ":class:`couchbase.transcoder.Transcoder` "
+                        ":class:`couchbase_core.transcoder.Transcoder` "
                         "is being used\n")
         },
 
@@ -576,13 +599,11 @@ static PyGetSetDef Bucket_TABLE_getset[] = {
                         "Note that this will still return true even if\n"
                         "it is subsequently closed via :meth:`_close`\n")
         },
-#ifdef PYCBC_TRACING
         {"tracer",
                 (getter) Bucket_tracer,
                 NULL,
                 PyDoc_STR("Tracer used by bucket, if any.\n")
         },
-#endif
         { "_instance_pointer",
                 (getter)Bucket__instance_pointer,
                 NULL,
@@ -701,6 +722,8 @@ static PyMethodDef Bucket_TABLE_methods[] = {
         OPFUNC(lock_multi, NULL),
         OPFUNC(_rget, NULL),
         OPFUNC(_rgetix, NULL),
+        OPFUNC(_rgetall, NULL),
+        OPFUNC(exists, "See if key exists in collection"),
 
         OPFUNC(mutate_in, "Perform mutations in document paths"),
         OPFUNC(lookup_in, "Perform lookups in document paths"),
@@ -721,7 +744,6 @@ static PyMethodDef Bucket_TABLE_methods[] = {
         OPFUNC(_n1ql_query, "Internal routine for N1QL queries"),
         OPFUNC(_cbas_query, "Internal routine for analytics queries"),
         OPFUNC(_fts_query, "Internal routine for Fulltext queries"),
-
         OPFUNC(_ixmanage, "Internal routine for managing indexes"),
         OPFUNC(_ixwatch, "Internal routine for monitoring indexes"),
 
@@ -730,7 +752,6 @@ static PyMethodDef Bucket_TABLE_methods[] = {
 
         OPFUNC(endure_multi, "Check durability requirements"),
 
-
 #undef OPFUNC
 
 
@@ -839,65 +860,179 @@ static PyMethodDef Bucket_TABLE_methods[] = {
                 PyDoc_STR("Decrypts a set of fields using the registered providers")
         },
 
-        { NULL, NULL, 0, NULL }
-};
+        {NULL, NULL, 0, NULL}};
 
-#ifdef PYCBC_TRACING
 void pycbc_Bucket_init_tracer(pycbc_Bucket *self);
-#endif
+
+lcb_STATUS pycbc_Collection_init_coords(pycbc_Collection_t *self,
+                                        pycbc_Bucket *bucket,
+                                        PyObject *collection,
+                                        PyObject *scope)
+{
+    lcb_STATUS err = LCB_SUCCESS;
+    self->bucket = bucket;
+    self->collection.scope = scope?pycbc_strn_from_managed(scope):(pycbc_strn_unmanaged){.content=pycbc_invalid_strn};
+    self->collection.collection = collection?pycbc_strn_from_managed(collection):(pycbc_strn_unmanaged){.content=pycbc_invalid_strn};
+    return err;
+}
+
+void pycbc_Collection_free_unmanaged_contents(
+        const pycbc_Collection_t *collection)
+{
+    pycbc_strn_free(collection->collection.scope);
+    pycbc_strn_free(collection->collection.collection);
+    PYCBC_XDECREF(collection->bucket);
+}
+
+static void Collection_dtor(pycbc_Collection_t *collection)
+{
+    pycbc_Collection_free_unmanaged_contents(collection);
+    Py_TYPE(collection)->tp_free((PyObject*)collection);
+
+}
+
+int pycbc_collection_init_from_fn_args(pycbc_Collection_t *self,
+                                       pycbc_Bucket *bucket,
+                                       PyObject *kwargs)
+{
+    int rv = LCB_SUCCESS;
+    PyObject *scope = NULL;
+    PyObject *collection = NULL;
+
+    PYCBC_EXCEPTION_LOG_NOCLEAR
+    if (!self->bucket) {
+        self->bucket = bucket;
+    }
+    PYCBC_XINCREF(self->bucket);
+    if (!kwargs) {
+        return LCB_SUCCESS;
+    }
+    PYCBC_DEBUG_PYFORMAT("Got kwargs %R", kwargs)
+    scope = kwargs ? PyDict_GetItemString(kwargs, "scope") : NULL;
+    collection = kwargs ? PyDict_GetItemString(kwargs, "collection") : NULL;
+
+    if (scope && collection) {
+        pycbc_Collection_init_coords(self, bucket, collection, scope);
+    }
+    if (scope) {
+        rv = PyDict_DelItemString(kwargs, "scope");
+        if (rv) {
+            PYCBC_DEBUG_LOG("Failed to delete from %S", kwargs);
+            PYCBC_EXCEPTION_LOG_NOCLEAR;
+        }
+    }
+    if (collection) {
+        PyDict_DelItemString(kwargs, "collection");
+    }
+    PYCBC_DEBUG_PYFORMAT("Kwargs are now %S", kwargs);
+    if (PyErr_Occurred()) {
+        PYCBC_DEBUG_LOG("Problems")
+        rv = LCB_ERR_COLLECTION_NOT_FOUND;
+        PYCBC_EXCEPTION_LOG_NOCLEAR
+    }
+    return rv;
+}
+
+static int Collection__init__(pycbc_Collection_t *self,
+                              PyObject *args,
+                              PyObject *kwargs)
+{
+    int rv = 0;
+    PyObject *maybe_bucket =
+            (PyObject_IsInstance(args, (PyObject *)&PyTuple_Type) &&
+             PyTuple_Size(args) > 0)
+                    ? PyTuple_GetItem(args, 0)
+                    : NULL;
+    if (!(maybe_bucket &&
+        PyObject_IsInstance(maybe_bucket, (PyObject *)&BucketType))) {
+        PYCBC_EXC_WRAP(PYCBC_EXC_ARGUMENTS,
+                       0,
+                       "First argument must be the parent bucket")
+        return -1;
+    }
+
+    rv = pycbc_collection_init_from_fn_args(
+            self, (pycbc_Bucket *)maybe_bucket, kwargs);
+
+    if (!rv) {
+        PYCBC_EXCTHROW_ARGS();
+        return -1;
+    }
+    return 0;
+}
+
+static PyMethodDef Collection_TABLE_methods[] = {{NULL, NULL, 0, NULL}};
+
+static PyGetSetDef Collection_TABLE_getset[] = {
+        {NULL}};
+
+static struct PyMemberDef Collection_TABLE_members[] = {{NULL}};
+
+int pycbc_CollectionType_init(PyObject **ptr)
+{
+    PyTypeObject *p = &BucketType;
+    *ptr = (PyObject *)p;
+
+    if (p->tp_name) {
+        return 0;
+    }
+
+    p->tp_name = "Collection";
+    p->tp_new = PyType_GenericNew;
+    p->tp_init = (initproc)Collection__init__;
+    p->tp_dealloc = (destructor)Collection_dtor;
+
+    p->tp_flags = Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE;
+    p->tp_doc = PyDoc_STR("The collection object");
+
+    p->tp_basicsize = sizeof(pycbc_Collection_t);
+
+    p->tp_methods = Collection_TABLE_methods;
+    p->tp_members = Collection_TABLE_members;
+    p->tp_getset = Collection_TABLE_getset;
+
+    pycbc_DummyTuple = PyTuple_New(0);
+    pycbc_DummyKeywords = PyDict_New();
+
+    return PyType_Ready(p);
+}
 
 static int
 Bucket__init__(pycbc_Bucket *self,
-                       PyObject *args, PyObject *kwargs)
+               PyObject *args, PyObject *kwargs)
 {
     int rv;
-    int conntype = LCB_TYPE_BUCKET;
 
-    lcb_error_t err;
-    PyObject *unlock_gil_O = NULL;
-    PyObject *iops_O = NULL;
-    PyObject *dfl_fmt = NULL;
-    PyObject *tc = NULL;
-    struct lcb_create_st create_opts = { 0 };
+    lcb_STATUS err=LCB_SUCCESS;
 
     /**
      * This xmacro enumerates the constructor keywords, targets, and types.
      * This was converted into an xmacro to ease the process of adding or
      * removing various parameters.
      */
-#define XCTOR_ARGS_NOTRACING(X)                                      \
-    X("connection_string", &create_opts.v.v3.connstr, "z") \
-    X("connstr", &create_opts.v.v3.connstr, "z")           \
-    X("username", &create_opts.v.v3.username, "z")         \
-    X("password", &create_opts.v.v3.passwd, "z")           \
-    X("quiet", &self->quiet, "I")                          \
-    X("unlock_gil", &unlock_gil_O, "O")                    \
-    X("transcoder", &tc, "O")                              \
-    X("default_format", &dfl_fmt, "O")                     \
-    X("lockmode", &self->lockmode, "i")                    \
-    X("_flags", &self->flags, "I")                         \
-    X("_conntype", &conntype, "i")                         \
-    X("_iops", &iops_O, "O")                               \
 
-#ifdef PYCBC_TRACING
-#define XCTOR_ARGS(X)\
-    XCTOR_ARGS_NOTRACING(X)\
-    X("tracer", &self->parent_tracer, "O")
-#else
-#define XCTOR_ARGS(X)\
-    XCTOR_ARGS_NOTRACING(X)
-#endif
+#define XCTOR_CREATEOPTS(CLASS)         \
+    CLASS##_TYPEOP(type, _conntype)     \
+    CLASS##_BUILD_CRED(                 \
+        CLASS##_CREDENTIAL(username),   \
+        CLASS##_CREDENTIAL(password))   \
+    CLASS##_STRING(connstr)\
+    CLASS##_STRING(bucket)
 
-    static char *kwlist[] = {
-        #define X(s, target, type) s,
-            XCTOR_ARGS(X)
-        #undef X
-            NULL
-    };
+#define XCTOR_NON_CREATEOPTS(CLASS)                                         \
+    CLASS##_STRINGALIAS(connection_string, connstr)                         \
+    CLASS##_TARGET_UINT(*self,quiet)                                        \
+    CLASS##_OBJECT(unlock_gil)                                              \
+    CLASS##_OBJECT(transcoder)                                              \
+    CLASS##_OBJECT(default_format)                                          \
+    CLASS##_TARGET_INT(*self, lockmode)                                     \
+    CLASS##_TARGET_UINTALIAS(*self, _flags, flags)                          \
+    CLASS##_OBJECT(_iops)                                                   \
+    CLASS##_TARGET_OBJECTALIAS(*self, tracer, parent_tracer)
 
-    #define X(s, target, type) type
-    static char *argspec = "|" XCTOR_ARGS(X);
-    #undef X
+#define XCTOR_ARGS(CLASS)       \
+    XCTOR_NON_CREATEOPTS(CLASS) \
+    XCTOR_CREATEOPTS(CLASS)
 
     if (self->init_called) {
         PyErr_SetString(PyExc_RuntimeError, "__init__ was already called");
@@ -908,63 +1043,70 @@ Bucket__init__(pycbc_Bucket *self,
     self->flags = 0;
     self->unlock_gil = 1;
     self->lockmode = PYCBC_LOCKMODE_EXC;
+    {
+        PYCBC_KWLIST(XCTOR_ARGS, opts)
 
-    #define X(s, target, type) target,
-    rv = PyArg_ParseTupleAndKeywords(args, kwargs, argspec, kwlist,
-        XCTOR_ARGS(X) NULL);
-    #undef X
+        if (opts.unlock_gil && PyObject_IsTrue(opts.unlock_gil) == 0) {
+            self->unlock_gil = 0;
+        }
 
-    if (!rv) {
-        PYCBC_EXCTHROW_ARGS();
-        return -1;
-    }
+        if (opts._iops && opts._iops != Py_None) {
+            self->iopswrap = pycbc_iowrap_new(self, opts._iops);
+            self->unlock_gil = 0;
+        }
 
-    if (unlock_gil_O && PyObject_IsTrue(unlock_gil_O) == 0) {
-        self->unlock_gil = 0;
-    }
-    create_opts.version = 3;
-    create_opts.v.v3.type = conntype;
+        if (opts.default_format == Py_None || opts.default_format == NULL) {
+            /** Set to 0 if None or NULL */
+            opts.default_format = pycbc_IntFromL(PYCBC_FMT_JSON);
 
-    if (iops_O && iops_O != Py_None) {
-        self->iopswrap = pycbc_iowrap_new(self, iops_O);
-        create_opts.v.v3.io = pycbc_iowrap_getiops(self->iopswrap);
-        self->unlock_gil = 0;
-    }
+        } else {
+            Py_INCREF(opts.default_format); /* later decref */
+        }
 
-    if (dfl_fmt == Py_None || dfl_fmt == NULL) {
-        /** Set to 0 if None or NULL */
-        dfl_fmt = pycbc_IntFromL(PYCBC_FMT_JSON);
+        rv = Bucket_set_format(self, opts.default_format, NULL);
+        Py_XDECREF(opts.default_format);
+        if (rv == -1) {
+            return rv;
+        }
 
-    } else {
-        Py_INCREF(dfl_fmt); /* later decref */
-    }
-
-    rv = Bucket_set_format(self, dfl_fmt, NULL);
-    Py_XDECREF(dfl_fmt);
-    if (rv == -1) {
-        return rv;
-    }
-
-    /** Set the transcoder */
-    if (tc && Bucket_set_transcoder(self, tc, NULL) == -1) {
-        return -1;
-    }
+        /** Set the transcoder */
+        if (opts.transcoder &&
+            Bucket_set_transcoder(self, opts.transcoder, NULL) == -1) {
+            return -1;
+        }
 
 #if defined(WITH_THREAD)
-    if (!self->unlock_gil) {
-        self->lockmode = PYCBC_LOCKMODE_NONE;
-    }
+        if (!self->unlock_gil) {
+            self->lockmode = PYCBC_LOCKMODE_NONE;
+        }
 
-    if (self->lockmode != PYCBC_LOCKMODE_NONE) {
-        self->lock = PyThread_allocate_lock();
-    }
+        if (self->lockmode != PYCBC_LOCKMODE_NONE) {
+            self->lock = PyThread_allocate_lock();
+        }
 #else
-    self->unlock_gil = 0;
-    self->lockmode = PYCBC_LOCKMODE_NONE;
+        self->unlock_gil = 0;
+        self->lockmode = PYCBC_LOCKMODE_NONE;
 #endif
+        {
+            lcb_CREATEOPTS *lcb_create_opts = NULL;
 
-    err = lcb_create(&self->instance, &create_opts);
+#define CREATE_TYPEOP(X, NAME) lcb_createopts_create(&lcb_create_opts, opts.X);
+#define CREATE_CREDENTIAL(X) opts.X, opts.X##_len
+#define CREATE_STRING(X) \
+    lcb_createopts_##X(lcb_create_opts, opts.X, opts.X##_len);
+#define CREATE_BUILD_CRED(USERNAME, PASSWORD) \
+    lcb_createopts_credentials(lcb_create_opts, USERNAME, PASSWORD);
+#define BUILD_OPTS(...) lcb_createopts_create(&lcb_create_opts, opts.type);
+            XCTOR_CREATEOPTS(CREATE);
 
+            if (self->iopswrap) {
+                lcb_createopts_io(lcb_create_opts,
+                                  pycbc_iowrap_getiops(self->iopswrap));
+            }
+            err = lcb_create(&self->instance, lcb_create_opts);
+            lcb_createopts_destroy(lcb_create_opts);
+        }
+    }
     if (err != LCB_SUCCESS) {
         self->instance = NULL;
         PYCBC_EXC_WRAP(PYCBC_EXC_LCBERR, err,
@@ -976,8 +1118,10 @@ Bucket__init__(pycbc_Bucket *self,
     }
 
     if (pycbc_log_handler) {
-        err = lcb_cntl(self->instance, LCB_CNTL_SET, LCB_CNTL_LOGGER,
-                       &pycbc_lcb_logprocs);
+        err = lcb_cntl(self->instance,
+                       LCB_CNTL_SET,
+                       LCB_CNTL_LOGGER,
+                       pycbc_lcb_logger);
         if (err != LCB_SUCCESS) {
             self->instance = NULL;
             PYCBC_EXC_WRAP(PYCBC_EXC_LCBERR, err, "Couldn't create log handler");
@@ -992,14 +1136,14 @@ Bucket__init__(pycbc_Bucket *self,
         err = lcb_cntl(self->instance, LCB_CNTL_GET, LCB_CNTL_BUCKETNAME, &bucketstr);
         if (err == LCB_SUCCESS && bucketstr != NULL) {
             self->bucket = pycbc_SimpleStringZ(bucketstr);
+        } else {
+            self->bucket = pycbc_SimpleStringZ("");
         }
     }
 
     self->btype = pycbc_IntFromL(LCB_BTYPE_UNSPEC);
-#ifdef PYCBC_TRACING
 
     pycbc_Bucket_init_tracer(self);
-#endif
 
     return 0;
 }
@@ -1009,7 +1153,6 @@ PyObject *pycbc_value_or_none_incref(PyObject *maybe_value)
     PYCBC_INCREF(result);
     return result;
 }
-#ifdef PYCBC_TRACING
 void pycbc_Bucket_init_tracer(pycbc_Bucket *self)
 {
     lcbtrace_TRACER *threshold_tracer = lcb_get_tracer(self->instance);
@@ -1040,11 +1183,11 @@ void pycbc_Bucket_init_tracer(pycbc_Bucket *self)
         PYCBC_DECREF(tracer_args);
     }
 }
-#endif
+
 static PyObject*
 Bucket__connect(pycbc_Bucket *self, PyObject* args, PyObject* kwargs)
 {
-    lcb_error_t err;
+    lcb_STATUS err=LCB_SUCCESS;
 
     if (self->flags & PYCBC_CONN_F_CONNECTED) {
         Py_RETURN_NONE;
@@ -1108,10 +1251,8 @@ Bucket_dtor(pycbc_Bucket *self)
     if (self->instance) {
         lcb_destroy(self->instance);
     }
-#ifdef PYCBC_TRACING
     PYCBC_XDECREF((PyObject*)self->tracer);
     self->tracer = NULL;
-#endif
 
 #ifdef WITH_THREAD
     if (self->lock) {
@@ -1123,8 +1264,7 @@ Bucket_dtor(pycbc_Bucket *self)
     Py_TYPE(self)->tp_free((PyObject*)self);
 }
 
-int
-pycbc_BucketType_init(PyObject **ptr)
+int pycbc_BucketType_init(PyObject **ptr)
 {
     PyTypeObject *p = &BucketType;
     *ptr = (PyObject*)p;
@@ -1133,7 +1273,7 @@ pycbc_BucketType_init(PyObject **ptr)
         return 0;
     }
 
-    p->tp_name = "Bucket";
+    p->tp_name = "Client";
     p->tp_new = PyType_GenericNew;
     p->tp_init = (initproc)Bucket__init__;
     p->tp_dealloc = (destructor)Bucket_dtor;
