@@ -26,21 +26,22 @@ CryptoProvider__init(pycbc_CryptoProvider *self,
 static void
 CryptoProvider_dtor(pycbc_CryptoProvider *self);
 
-lcb_error_t pycbc_cstrndup(char **key, size_t *key_len, PyObject *result)
+lcb_STATUS pycbc_cstrndup(char **key, size_t *key_len, PyObject *result)
 {
     const char *data = NULL;
-    lcb_error_t lcb_result = LCB_EINTERNAL;
+    lcb_STATUS lcb_result = LCB_ERR_SDK_INTERNAL;
 
     data = PYCBC_CSTRN(result, key_len);;
     if (data) {
         lcb_result = LCB_SUCCESS;
-        PYCBC_DEBUG_LOG("Got string from %p: %.*s", result, (int)key_len, data);
+        PYCBC_DEBUG_LOG(
+                "Got string from %p: %.*s", result, (int)*key_len, data);
         *key = calloc(1, *key_len + 1);
         memcpy((void *)*key, (void *)data, *key_len);
         (*key)[*key_len] = '\0';
         PYCBC_DEBUG_LOG("Copied string from %p: %.*s",
                         result,
-                        (int)key_len,
+                        (int)*key_len,
                         (char *)*key);
     } else {
         PYCBC_DEBUG_PYFORMAT(
@@ -57,21 +58,21 @@ const char *pycbc_cstrdup_or_default_and_exception(
 
     if (!buffer_data) {
         PYCBC_EXC_WRAP(PYCBC_EXC_LCBERR,
-                       LCB_GENERIC_TMPERR,
+                       LCB_ERR_TEMPORARY_FAILURE,
                        "CryptoProviderMissingPublicKeyException")
     }
     return buffer_data ? buffer_data : fallback;
 }
 
-lcb_error_t pycbc_is_true(uint8_t *key, const size_t key_len, PyObject *result) {
-    return (result && PyObject_IsTrue(result) && !PyErr_Occurred())?
-           LCB_SUCCESS:
-           LCB_EINTERNAL;
+lcb_STATUS pycbc_is_true(uint8_t *key, const size_t key_len, PyObject *result) {
+    return (result && PyObject_IsTrue(result) && !PyErr_Occurred())
+                   ? LCB_SUCCESS
+                   : LCB_ERR_SDK_INTERNAL;
 }
 
 #define PYCBC_SIZE(X) (sizeof(X)/sizeof(char))
 
-void pycbc_report_method_exception(lcb_error_t errflags, const char* fmt, ...) {
+void pycbc_report_method_exception(lcb_STATUS errflags, const char* fmt, ...) {
     char buffer[1000];
     va_list args;
     va_start(args,fmt);
@@ -88,7 +89,9 @@ static PyObject* pycbc_retrieve_method(lcbcrypto_PROVIDER* provider, const char*
                                                         method_name)
                                : NULL;
     if (!method || !PyObject_IsTrue(method)) {
-        pycbc_report_method_exception(LCB_GENERIC_TMPERR, "Method %s does not exist", method_name);
+        pycbc_report_method_exception(LCB_ERR_TEMPORARY_FAILURE,
+                                      "Method %s does not exist",
+                                      method_name);
         return NULL;
     }
     PYCBC_DEBUG_LOG("Got method pointer %p for %s", method, method_name);
@@ -107,10 +110,10 @@ PyObject* pycbc_python_proxy(PyObject *method, PyObject *args, const char* metho
     };
     result = PyObject_CallObject(method, args);
     PYCBC_DEBUG_PYFORMAT("Called %R with %R, got %p", method, args, result);
-    PYCBC_DEBUG_PYFORMAT("%p is %S", result, result);
+    PYCBC_DEBUG_PYFORMAT("%p is %S", result, pycbc_none_or_value(result));
     if (!result || PyErr_Occurred()) {
-        pycbc_report_method_exception(LCB_EINTERNAL, "Problem calling method %s",
-                                      method_name);
+        pycbc_report_method_exception(
+                LCB_ERR_SDK_INTERNAL, "Problem calling method %s", method_name);
         result =NULL;
     };
 
@@ -158,11 +161,6 @@ PyObject *pycbc_convert_lcbcrypto_SIGV(const lcbcrypto_SIGV *sigv) {
     return pycbc_convert_uint8_t(buf);
 }
 
-PyObject* pycbc_convert_lcbcrypto_KEYTYPE(const lcbcrypto_KEYTYPE type)
-{
-    return PyLong_FromLong(type);
-}
-
 PyObject* pycbc_convert_char_p(const char* string)
 {
     return pycbc_SimpleStringZ(string);
@@ -246,7 +244,7 @@ pycbc_crypto_buf pycbc_gen_buf(const uint8_t *data, size_t len)
 
 typedef const char *PYCBC_CSTR_T;
 PYCBC_CSTR_T PYCBC_CSTR_T_ERRVALUE = "[VALUE NOT FOUND]";
-lcb_error_t lcb_error_t_ERRVALUE = LCB_GENERIC_TMPERR;
+lcb_STATUS lcb_STATUS_ERRVALUE = LCB_ERR_TEMPORARY_FAILURE;
 
 #define PYCBC_CSTRDUP_WRAPPER(DUMMY, DUMMY2, OBJECT) \
     pycbc_cstrdup_or_default_and_exception(          \
@@ -255,22 +253,22 @@ lcb_error_t lcb_error_t_ERRVALUE = LCB_GENERIC_TMPERR;
 #define PYCBC_CSTRNDUP_WRAPPER(BUF, BUF_LEN, OBJECT) \
     pycbc_cstrndup((char **)BUF, BUF_LEN, OBJECT)
 
-#define EXC(X) ((lcb_error_t)X),((lcb_error_t)PYCBC_CRYPTO_PROVIDER_KEY_SIZE_EXCEPTION),
+#define EXC(X) ((lcb_STATUS)X),((lcb_STATUS)PYCBC_CRYPTO_PROVIDER_KEY_SIZE_EXCEPTION),
 
 #define PYCBC_X_COMMON_CRYPTO_METHODS(X)         \
-    X(lcb_error_t,                               \
+    X(lcb_STATUS,                               \
       generic,                                   \
       generate_iv,                               \
       PYCBC_CSTRNDUP_WRAPPER,                    \
       PYCBC_GENERATE_IV_TYPES,                   \
       EXC(PYCBC_CRYPTO_EXECUTION_ERROR))         \
-    X(lcb_error_t,                               \
+    X(lcb_STATUS,                               \
       generic,                                   \
       sign,                                      \
       PYCBC_CSTRNDUP_WRAPPER,                    \
       PYCBC_SIGN_TYPES,                          \
       EXC(PYCBC_CRYPTO_PROVIDER_SIGNING_FAILED)) \
-    X(lcb_error_t,                               \
+    X(lcb_STATUS,                               \
       generic,                                   \
       verify_signature,                          \
       pycbc_is_true,                             \
@@ -278,18 +276,18 @@ lcb_error_t lcb_error_t_ERRVALUE = LCB_GENERIC_TMPERR;
       EXC(PYCBC_CRYPTO_ERROR))
 
 #define PYCBC_X_V0_ONLY_CRYPTO_METHODS(X) \
-    X(lcb_error_t,                        \
+    X(lcb_STATUS,                        \
       v0,                                 \
       load_key,                           \
       PYCBC_CSTRNDUP_WRAPPER,             \
       PYCBC_LOAD_KEY_TYPES,               \
       EXC(PYCBC_CRYPTO_ERROR))            \
-    X(lcb_error_t,                        \
+    X(lcb_STATUS,                        \
       v0,                                 \
       encrypt,                            \
       PYCBC_CSTRNDUP_WRAPPER,             \
       PYCBC_V0_ENCRYPT_TYPES)             \
-    X(lcb_error_t,                        \
+    X(lcb_STATUS,                        \
       v0,                                 \
       decrypt,                            \
       PYCBC_CSTRNDUP_WRAPPER,             \
@@ -301,13 +299,13 @@ PYCBC_X_V0_ONLY_CRYPTO_METHODS(X)\
 PYCBC_X_COMMON_CRYPTO_METHODS(X)
 
 #define PYCBC_X_V1_ONLY_CRYPTO_METHODS(X)        \
-    X(lcb_error_t,                               \
+    X(lcb_STATUS,                               \
       v1,                                        \
       encrypt,                                   \
       PYCBC_CSTRNDUP_WRAPPER,                    \
       PYCBC_V1_ENCRYPT_TYPES,                    \
       EXC(PYCBC_CRYPTO_PROVIDER_ENCRYPT_FAILED)) \
-    X(lcb_error_t,                               \
+    X(lcb_STATUS,                               \
       v1,                                        \
       decrypt,                                   \
       PYCBC_CSTRNDUP_WRAPPER,                    \
@@ -326,7 +324,6 @@ PYCBC_X_COMMON_CRYPTO_METHODS(X)
 
 #define PYCBC_X_ALL_CRYPTO_FUNCTIONS(X) \
     PYCBC_X_COMMON_CRYPTO_METHODS(X)    \
-    PYCBC_X_V0_ONLY_CRYPTO_METHODS(X)   \
     PYCBC_X_V1_ONLY_CRYPTO_METHODS(X)
 
 #define PYCBC_SIG_METHOD(RTYPE, VERSION, METHOD, PROCESSOR, X_ARGS, ...)       \
@@ -385,7 +382,7 @@ pycbc_NamedCryptoProvider *pycbc_extract_named_crypto_provider(
 }
 
 void pycbc_exc_wrap_obj(pycbc_NamedCryptoProvider *named_crypto_provider,
-                        lcb_error_t err_code)
+                        lcb_STATUS err_code)
 {
     PyObject *name =
             named_crypto_provider
@@ -402,9 +399,9 @@ void pycbc_exc_wrap_obj(pycbc_NamedCryptoProvider *named_crypto_provider,
     //Py_DecRef(attrib_dict);
 }
 
-#if PYCBC_CRYPTO_VERSION == 1
-#define PYCBC_CRYPTO_VVERSION v1
-#define PYCBC_CRYPTO_METHODS(X) PYCBC_X_V1_CRYPTO_METHODS(X)
+#if PYCBC_CRYPTO_VERSION > 0
+#    define PYCBC_CRYPTO_VVERSION v1
+#    define PYCBC_CRYPTO_METHODS(X) PYCBC_X_V1_CRYPTO_METHODS(X)
 #else
 #define PYCBC_CRYPTO_VVERSION v0
 #define PYCBC_CRYPTO_METHODS(X) PYCBC_X_V0_CRYPTO_METHODS(X)
@@ -441,18 +438,18 @@ void pycbc_exc_wrap_obj(pycbc_NamedCryptoProvider *named_crypto_provider,
         }                                                                     \
         if (lcb_result == RTYPE##_ERRVALUE) {                                 \
             PYCBC_WRAP_CRYPTO_EXCEPTION(named_crypto_provider,                \
-                                        __VA_ARGS__ lcb_error_t_ERRVALUE);    \
+                                        __VA_ARGS__ lcb_STATUS_ERRVALUE);    \
         }                                                                     \
     FAIL:                                                                     \
         return lcb_result;                                                    \
     }
 
-PyObject *pycbc_va_list_v(lcb_error_t sentinel, va_list errs)
+PyObject *pycbc_va_list_v(lcb_STATUS sentinel, va_list errs)
 {
     PyObject *err_list = PyList_New(0);
     do {
         PyObject *py_enum;
-        lcb_error_t val = va_arg(errs, lcb_error_t);
+        lcb_STATUS val = va_arg(errs, lcb_STATUS);
         if (val == sentinel)
             break;
         py_enum = PyLong_FromLong(val);
@@ -464,7 +461,7 @@ PyObject *pycbc_va_list_v(lcb_error_t sentinel, va_list errs)
 
 void pycbc_set_var_items_dict(PyObject *dict,
                               const char *key,
-                              lcb_error_t sentinel,
+                              lcb_STATUS sentinel,
                               ...)
 {
     PyObject *err_list;
@@ -483,8 +480,8 @@ PyObject *pycbc_gen_crypto_exception_map(void)
         RTYPE, VERSION, METHOD, PROCESSOR, X_ARGS, ...)                    \
     pycbc_set_var_items_dict(exception_map,                                \
                              #METHOD,                                      \
-                             (lcb_error_t)PYCBC_CRYPTO_PROVIDER_ERROR_MAX, \
-                             __VA_ARGS__(lcb_error_t)                      \
+                             (lcb_STATUS)PYCBC_CRYPTO_PROVIDER_ERROR_MAX, \
+                             __VA_ARGS__(lcb_STATUS)                      \
                                      PYCBC_CRYPTO_PROVIDER_ERROR_MAX);
     PYCBC_CRYPTO_METHODS(PYCBC_CRYPTO_EXC_MAP_WRAPPER)
 
@@ -496,9 +493,9 @@ PYCBC_X_ALL_CRYPTO_FUNCTIONS(PYCBC_SIG_METHOD)
 PYCBC_CRYPTO_METHODS(PYCBC_CRYPTO_EXC_WRAPPER)
 #else
 
-static lcb_error_t pycbc_crypto_generic_generate_iv(lcbcrypto_PROVIDER *provider, uint8_t **subject,
+static lcb_STATUS pycbc_crypto_generic_generate_iv(lcbcrypto_PROVIDER *provider, uint8_t **subject,
                                                     size_t *subject_len) {
-    lcb_error_t lcb_result = lcb_error_t_ERRVALUE;
+    lcb_STATUS lcb_result = lcb_STATUS_ERRVALUE;
     PyObject *method = !PyErr_Occurred() ? pycbc_retrieve_method(provider, "generate_iv") : ((void *) 0);
     if (method) {
         const char *PYARGS_FMTSTRING = "(" ")";
@@ -513,10 +510,10 @@ static lcb_error_t pycbc_crypto_generic_generate_iv(lcbcrypto_PROVIDER *provider
     return lcb_result;
 }
 
-static lcb_error_t
+static lcb_STATUS
 pycbc_crypto_generic_sign(lcbcrypto_PROVIDER *provider, const lcbcrypto_SIGV *inputs, size_t inputs_num,
                           uint8_t **subject, size_t *subject_len) {
-    lcb_error_t lcb_result = lcb_error_t_ERRVALUE;
+    lcb_STATUS lcb_result = lcb_STATUS_ERRVALUE;
     PyObject *method = !PyErr_Occurred() ? pycbc_retrieve_method(provider, "sign") : ((void *) 0);
     if (method) {
         PyObject *inputs_list = pycbc_gen_list_lcbcrypto_SIGV(inputs, inputs_num, pycbc_convert_lcbcrypto_SIGV);
@@ -533,10 +530,10 @@ pycbc_crypto_generic_sign(lcbcrypto_PROVIDER *provider, const lcbcrypto_SIGV *in
     return lcb_result;
 }
 
-static lcb_error_t
+static lcb_STATUS
 pycbc_crypto_generic_verify_signature(lcbcrypto_PROVIDER *provider, const lcbcrypto_SIGV *inputs, size_t inputs_num,
                                       uint8_t *subject, size_t subject_len) {
-    lcb_error_t lcb_result = lcb_error_t_ERRVALUE;
+    lcb_STATUS lcb_result = lcb_STATUS_ERRVALUE;
     PyObject *method = !PyErr_Occurred() ? pycbc_retrieve_method(provider, "verify_signature") : ((void *) 0);
     if (method) {
         PyObject *inputs_list = pycbc_gen_list_lcbcrypto_SIGV(inputs, inputs_num, pycbc_convert_lcbcrypto_SIGV);
@@ -553,78 +550,11 @@ pycbc_crypto_generic_verify_signature(lcbcrypto_PROVIDER *provider, const lcbcry
     return lcb_result;
 }
 
-static lcb_error_t
-pycbc_crypto_v0_load_key(lcbcrypto_PROVIDER *provider, lcbcrypto_KEYTYPE type, const char *keyid, uint8_t **subject,
-                         size_t *subject_len) {
-    lcb_error_t lcb_result = lcb_error_t_ERRVALUE;
-    PyObject *method = !PyErr_Occurred() ? pycbc_retrieve_method(provider, "load_key") : ((void *) 0);
-    if (method) {
-        PyObject *keyid_converted = pycbc_convert_char_p(keyid);
-        const char *PYARGS_FMTSTRING = "(" "O" "O" ")";
-        PyObject *args = Py_BuildValue(PYARGS_FMTSTRING, pycbc_convert_lcbcrypto_KEYTYPE(type), keyid_converted);
-        PyObject *result = pycbc_python_proxy(method, args, "load_key");
-        if (result) {
-            lcb_result = pycbc_cstrndup((char **)subject, subject_len, result);
-        }
-        Py_DecRef(result);
-        Py_DecRef(args);
-        Py_DecRef(keyid_converted);
-    }
-    return lcb_result;
-}
 
-static lcb_error_t
-pycbc_crypto_v0_encrypt(lcbcrypto_PROVIDER *provider, const uint8_t *input, size_t input_len, const uint8_t *key,
-                        size_t key_len, const uint8_t *iv, size_t iv_len, uint8_t **subject, size_t *subject_len) {
-    lcb_error_t lcb_result = lcb_error_t_ERRVALUE;
-    PyObject *method = !PyErr_Occurred() ? pycbc_retrieve_method(provider, "encrypt") : ((void *) 0);
-    if (method) {
-        PyObject *input_cstrn = pycbc_convert_uint8_t(pycbc_gen_buf(input, (input_len > 0) ? (input_len - 1) : 0));
-        PyObject *key_cstrn = pycbc_convert_uint8_t(pycbc_gen_buf(key, key_len));
-        PyObject *iv_cstrn = pycbc_convert_uint8_t(pycbc_gen_buf(iv, iv_len));
-        const char *PYARGS_FMTSTRING = "(" "O" "O" "O" ")";
-        PyObject *args = Py_BuildValue(PYARGS_FMTSTRING, input_cstrn, key_cstrn, iv_cstrn);
-        PyObject *result = pycbc_python_proxy(method, args, "encrypt");
-        if (result) {
-            lcb_result = pycbc_cstrndup((char **)subject, subject_len, result);
-        }
-        Py_DecRef(result);
-        Py_DecRef(args);
-        Py_DecRef(input_cstrn);
-        Py_DecRef(key_cstrn);
-        Py_DecRef(iv_cstrn);
-    }
-    return lcb_result;
-}
-
-static lcb_error_t
-pycbc_crypto_v0_decrypt(lcbcrypto_PROVIDER *provider, const uint8_t *input, size_t input_len, const uint8_t *key,
-                        size_t key_len, const uint8_t *iv, size_t iv_len, uint8_t **subject, size_t *subject_len) {
-    lcb_error_t lcb_result = lcb_error_t_ERRVALUE;
-    PyObject *method = !PyErr_Occurred() ? pycbc_retrieve_method(provider, "decrypt") : ((void *) 0);
-    if (method) {
-        PyObject *input_cstrn = pycbc_convert_uint8_t(pycbc_gen_buf(input, input_len));
-        PyObject *key_cstrn = pycbc_convert_uint8_t(pycbc_gen_buf(key, key_len));
-        PyObject *iv_cstrn = pycbc_convert_uint8_t(pycbc_gen_buf(iv, iv_len));
-        const char *PYARGS_FMTSTRING = "(" "O" "O" "O" ")";
-        PyObject *args = Py_BuildValue(PYARGS_FMTSTRING, input_cstrn, key_cstrn, iv_cstrn);
-        PyObject *result = pycbc_python_proxy(method, args, "decrypt");
-        if (result) {
-            lcb_result = pycbc_cstrndup((char **)subject, subject_len, result);
-        }
-        Py_DecRef(result);
-        Py_DecRef(args);
-        Py_DecRef(input_cstrn);
-        Py_DecRef(key_cstrn);
-        Py_DecRef(iv_cstrn);
-    }
-    return lcb_result;
-}
-
-static lcb_error_t
+static lcb_STATUS
 pycbc_crypto_v1_encrypt(lcbcrypto_PROVIDER *provider, const uint8_t *input, size_t input_len, const uint8_t *iv,
                         size_t iv_len, uint8_t **subject, size_t *subject_len) {
-    lcb_error_t lcb_result = lcb_error_t_ERRVALUE;
+    lcb_STATUS lcb_result = lcb_STATUS_ERRVALUE;
     PyObject *method = !PyErr_Occurred() ? pycbc_retrieve_method(provider, "encrypt") : ((void *) 0);
     if (method) {
         PyObject *input_cstrn = pycbc_convert_uint8_t(pycbc_gen_buf(input, (input_len > 0) ? (input_len - 1) : 0));
@@ -643,10 +573,10 @@ pycbc_crypto_v1_encrypt(lcbcrypto_PROVIDER *provider, const uint8_t *input, size
     return lcb_result;
 }
 
-static lcb_error_t
+static lcb_STATUS
 pycbc_crypto_v1_decrypt(lcbcrypto_PROVIDER *provider, const uint8_t *input, size_t input_len, const uint8_t *iv,
                         size_t iv_len, uint8_t **subject, size_t *subject_len) {
-    lcb_error_t lcb_result = lcb_error_t_ERRVALUE;
+    lcb_STATUS lcb_result = lcb_STATUS_ERRVALUE;
     PyObject *method = !PyErr_Occurred() ? pycbc_retrieve_method(provider, "decrypt") : ((void *) 0);
     if (method) {
         PyObject *input_cstrn = pycbc_convert_uint8_t(pycbc_gen_buf(input, input_len));
@@ -687,7 +617,7 @@ static PYCBC_CSTR_T pycbc_crypto_V1_get_key_id(lcbcrypto_PROVIDER *provider)
     return lcb_result;
 }
 
-static lcb_error_t pycbc_crypto_exc_wrap_v1_encrypt(
+static lcb_STATUS pycbc_crypto_exc_wrap_v1_encrypt(
         lcbcrypto_PROVIDER *provider,
         const uint8_t *input,
         size_t input_len,
@@ -700,7 +630,7 @@ static lcb_error_t pycbc_crypto_exc_wrap_v1_encrypt(
             pycbc_extract_named_crypto_provider(provider);
     lcbcrypto_PROVIDER *orig_lcb_provider =
             named_crypto_provider->orig_py_provider->lcb_provider;
-    lcb_error_t lcb_result = lcb_error_t_ERRVALUE;
+    lcb_STATUS lcb_result = lcb_STATUS_ERRVALUE;
     if (PyErr_Occurred()) {
         goto FAIL;
     }
@@ -713,15 +643,15 @@ static lcb_error_t pycbc_crypto_exc_wrap_v1_encrypt(
                                                      subject,
                                                      subject_len);
     }
-    if (lcb_result == lcb_error_t_ERRVALUE) {
+    if (lcb_result == lcb_STATUS_ERRVALUE) {
         pycbc_exc_wrap_obj(named_crypto_provider,
-                           ((lcb_error_t)PYCBC_CRYPTO_PROVIDER_ENCRYPT_FAILED));
+                           ((lcb_STATUS)PYCBC_CRYPTO_PROVIDER_ENCRYPT_FAILED));
     }
 FAIL:
     return lcb_result;
 }
 
-static lcb_error_t pycbc_crypto_exc_wrap_v1_decrypt(
+static lcb_STATUS pycbc_crypto_exc_wrap_v1_decrypt(
         lcbcrypto_PROVIDER *provider,
         const uint8_t *input,
         size_t input_len,
@@ -734,7 +664,7 @@ static lcb_error_t pycbc_crypto_exc_wrap_v1_decrypt(
             pycbc_extract_named_crypto_provider(provider);
     lcbcrypto_PROVIDER *orig_lcb_provider =
             named_crypto_provider->orig_py_provider->lcb_provider;
-    lcb_error_t lcb_result = lcb_error_t_ERRVALUE;
+    lcb_STATUS lcb_result = lcb_STATUS_ERRVALUE;
     if (PyErr_Occurred()) {
         goto FAIL;
     }
@@ -747,9 +677,9 @@ static lcb_error_t pycbc_crypto_exc_wrap_v1_decrypt(
                                                      subject,
                                                      subject_len);
     }
-    if (lcb_result == lcb_error_t_ERRVALUE) {
+    if (lcb_result == lcb_STATUS_ERRVALUE) {
         pycbc_exc_wrap_obj(named_crypto_provider,
-                           ((lcb_error_t)PYCBC_CRYPTO_PROVIDER_DECRYPT_FAILED));
+                           ((lcb_STATUS)PYCBC_CRYPTO_PROVIDER_DECRYPT_FAILED));
     }
 FAIL:
     return lcb_result;
@@ -771,20 +701,20 @@ static PYCBC_CSTR_T pycbc_crypto_exc_wrap_V1_get_key_id(
     }
     if (lcb_result == PYCBC_CSTR_T_ERRVALUE) {
         pycbc_exc_wrap_obj(named_crypto_provider,
-                           ((lcb_error_t)PYCBC_CRYPTO_ERROR));
+                           ((lcb_STATUS)PYCBC_CRYPTO_ERROR));
     }
 FAIL:
     return lcb_result;
 }
 
-static lcb_error_t pycbc_crypto_exc_wrap_generic_generate_iv(
+static lcb_STATUS pycbc_crypto_exc_wrap_generic_generate_iv(
         lcbcrypto_PROVIDER *provider, uint8_t **subject, size_t *subject_len)
 {
     pycbc_NamedCryptoProvider *named_crypto_provider =
             pycbc_extract_named_crypto_provider(provider);
     lcbcrypto_PROVIDER *orig_lcb_provider =
             named_crypto_provider->orig_py_provider->lcb_provider;
-    lcb_error_t lcb_result = lcb_error_t_ERRVALUE;
+    lcb_STATUS lcb_result = lcb_STATUS_ERRVALUE;
     if (PyErr_Occurred()) {
         goto FAIL;
     }
@@ -792,15 +722,15 @@ static lcb_error_t pycbc_crypto_exc_wrap_generic_generate_iv(
         lcb_result = orig_lcb_provider->v.v1.generate_iv(
                 orig_lcb_provider, subject, subject_len);
     }
-    if (lcb_result == lcb_error_t_ERRVALUE) {
+    if (lcb_result == lcb_STATUS_ERRVALUE) {
         pycbc_exc_wrap_obj(named_crypto_provider,
-                           ((lcb_error_t)PYCBC_CRYPTO_EXECUTION_ERROR));
+                           ((lcb_STATUS)PYCBC_CRYPTO_EXECUTION_ERROR));
     }
 FAIL:
     return lcb_result;
 }
 
-static lcb_error_t pycbc_crypto_exc_wrap_generic_sign(
+static lcb_STATUS pycbc_crypto_exc_wrap_generic_sign(
         lcbcrypto_PROVIDER *provider,
         const lcbcrypto_SIGV *inputs,
         size_t inputs_num,
@@ -811,7 +741,7 @@ static lcb_error_t pycbc_crypto_exc_wrap_generic_sign(
             pycbc_extract_named_crypto_provider(provider);
     lcbcrypto_PROVIDER *orig_lcb_provider =
             named_crypto_provider->orig_py_provider->lcb_provider;
-    lcb_error_t lcb_result = lcb_error_t_ERRVALUE;
+    lcb_STATUS lcb_result = lcb_STATUS_ERRVALUE;
     if (PyErr_Occurred()) {
         goto FAIL;
     }
@@ -819,15 +749,15 @@ static lcb_error_t pycbc_crypto_exc_wrap_generic_sign(
         lcb_result = orig_lcb_provider->v.v1.sign(
                 orig_lcb_provider, inputs, inputs_num, subject, subject_len);
     }
-    if (lcb_result == lcb_error_t_ERRVALUE) {
+    if (lcb_result == lcb_STATUS_ERRVALUE) {
         pycbc_exc_wrap_obj(named_crypto_provider,
-                           ((lcb_error_t)PYCBC_CRYPTO_PROVIDER_SIGNING_FAILED));
+                           ((lcb_STATUS)PYCBC_CRYPTO_PROVIDER_SIGNING_FAILED));
     }
 FAIL:
     return lcb_result;
 }
 
-static lcb_error_t pycbc_crypto_exc_wrap_generic_verify_signature(
+static lcb_STATUS pycbc_crypto_exc_wrap_generic_verify_signature(
         lcbcrypto_PROVIDER *provider,
         const lcbcrypto_SIGV *inputs,
         size_t inputs_num,
@@ -838,7 +768,7 @@ static lcb_error_t pycbc_crypto_exc_wrap_generic_verify_signature(
             pycbc_extract_named_crypto_provider(provider);
     lcbcrypto_PROVIDER *orig_lcb_provider =
             named_crypto_provider->orig_py_provider->lcb_provider;
-    lcb_error_t lcb_result = lcb_error_t_ERRVALUE;
+    lcb_STATUS lcb_result = lcb_STATUS_ERRVALUE;
     if (PyErr_Occurred()) {
         goto FAIL;
     }
@@ -846,9 +776,9 @@ static lcb_error_t pycbc_crypto_exc_wrap_generic_verify_signature(
         lcb_result = orig_lcb_provider->v.v1.verify_signature(
                 orig_lcb_provider, inputs, inputs_num, subject, subject_len);
     }
-    if (lcb_result == lcb_error_t_ERRVALUE) {
+    if (lcb_result == lcb_STATUS_ERRVALUE) {
         pycbc_exc_wrap_obj(named_crypto_provider,
-                           ((lcb_error_t)PYCBC_CRYPTO_ERROR));
+                           ((lcb_STATUS)PYCBC_CRYPTO_ERROR));
     }
 FAIL:
     return lcb_result;
@@ -894,7 +824,7 @@ void pycbc_release_bytes(lcbcrypto_PROVIDER *provider, void *bytes)
         Py_DecRef(method);                                                  \
     } else if (!self->lcb_provider->v.PYCBC_CRYPTO_VVERSION.METHOD) {       \
         pycbc_report_method_exception(                                      \
-                LCB_EINVAL, "Missing method %s", #METHOD);                  \
+                LCB_ERR_INVALID_ARGUMENT, "Missing method %s", #METHOD);    \
     }
 
 void pycbc_named_crypto_provider_destructor(lcbcrypto_PROVIDER *provider)
